@@ -134,7 +134,7 @@ def command(*args):
         stderr=subprocess.STDOUT,
     )
     ret = p.wait()
-    verbose = p.stdout.read().decode()
+    verbose = p.stdout.read().decode('utf-8')
     if ret != 0: raise Exception(verbose)
     return verbose
 
@@ -196,16 +196,18 @@ def win_get_interfaces():
             adNode = a.ipAddressList
             while True:
                 #convert ipAddr and ipMask into hex addrs that can be turned into a bcast addr
-                ipAddr = adNode.ipAddress.decode()
-                ipMask = adNode.ipMask.decode()
+                try:
+                    ipAddr = adNode.ipAddress.decode()
+                    ipMask = adNode.ipMask.decode()
+                except AttributeError: pass
                 if ipAddr and ipMask:
                     hexAddr = struct.unpack("<L", socket.inet_aton(ipAddr))[0]
                     hexMask = struct.unpack("<L", socket.inet_aton(ipMask))[0]
                     if(hexAddr and hexMask): #don't broadcast on 255.255.255.255, that's just lame
                         yield socket.inet_ntoa(struct.pack("<L", (hexAddr & hexMask) | (~hexMask) & 0xFFFFFFFF))
-                adNode = adNode.next
-                if not adNode:
-                    break
+                try: adNode = adNode.next
+                except: break
+                if not adNode: break
 
 def enumerate_devices():
     for bcast_addr in get_interfaces():
@@ -282,7 +284,7 @@ class burner_socket(object):
         (flash_size, sector_size) = self.get_flash_info()
         hw_rev = self.get_hw_rev()
 
-        if(hw_rev != 0): print("Hardware type: %s" % n2xx_revs[hw_rev][0])
+        if hw_rev in n2xx_revs: print("Hardware type: %s" % n2xx_revs[hw_rev][0])
         print("Flash size: %i\nSector size: %i\n" % (flash_size, sector_size))
 
         if fpga:
